@@ -1,8 +1,10 @@
 from urlparse import urlunsplit, urljoin
 from xml.dom import minidom
 import urllib
+from urlparse import urlparse
 import csv
 import base64
+import httplib
 
 # Use core json for 2.6+, simplejson for <=2.5
 try:
@@ -14,6 +16,7 @@ except ImportError:
 from .query import Query, Template
 from .model import Model
 from .util import ReadableException
+from .lists import ListManager
 
 """
 Webservice Interaction Routines for InterMine Webservices
@@ -81,16 +84,21 @@ class Service(object):
 
     @see: L{intermine.query}
     """
-    QUERY_PATH         = '/query/results'
-    MODEL_PATH         = '/model'
-    TEMPLATES_PATH     = '/templates/xml'
-    TEMPLATEQUERY_PATH = '/template/results'
-    VERSION_PATH       = '/version'
-    USER_AGENT         = 'WebserviceInterMinePerlAPIClient'
-    LIST_PATH          = '/lists/xml'
-    SAVEDQUERY_PATH    = '/savedqueries/xml'
-    RELEASE_PATH       = '/version/release'
-    SCHEME             = 'http://'
+    QUERY_PATH             = '/query/results'
+    QUERY_LIST_UPLOAD_PATH = '/query/tolist/json'
+    QUERY_LIST_APPEND_PATH = '/query/append/tolist/json'
+    MODEL_PATH             = '/model'
+    TEMPLATES_PATH         = '/templates/xml'
+    TEMPLATEQUERY_PATH     = '/template/results'
+    VERSION_PATH           = '/version'
+    USER_AGENT             = 'WebserviceInterMinePerlAPIClient'
+    LIST_PATH              = '/lists/json'
+    LIST_CREATION_PATH     = '/lists/json'
+    LIST_RENAME_PATH       = '/lists/rename/json'
+    LIST_APPENDING_PATH    = '/lists/append/json'
+    SAVEDQUERY_PATH        = '/savedqueries/xml'
+    RELEASE_PATH           = '/version/release'
+    SCHEME                 = 'http://'
 
     def __init__(self, root, username=None, password=None):
         """
@@ -113,9 +121,11 @@ class Service(object):
         self._model = None
         self._version = None
         self._release = None
+        self._list_manager = ListManager(self)
+        self.__missing_method_name = None
         if username:
             if not password:
-                raise ValueError("No password supplied")
+                raise ValueError("Username given, but no password supplied")
             self.opener = InterMineURLOpener((username, password))
         else:
             self.opener = InterMineURLOpener()
@@ -126,6 +136,30 @@ class Service(object):
 #       except ServiceError:
 #           raise ServiceError("Could not validate service - is the root url correct?")
 
+    # Delegated list methods
+
+    LIST_MANAGER_METHODS = frozenset(["get_list", "get_all_lists", "get_all_list_names",
+        "create_list", "get_list_count", "delete_lists"])
+
+    def __getattribute__(self, name):
+        return object.__getattribute__(self, name)
+    
+    def __getattr__(self, name):
+        self.__missing_method_name = name # Could also be a property
+        return getattr(self, '__methodmissing__')
+
+    def __methodmissing__(self, *args, **kwargs):
+        if self.__missing_method_name in self.LIST_MANAGER_METHODS:
+            method = getattr(self._list_manager, self.__missing_method_name)
+        self.__missing_method_name = None
+        return method(*args, **kwargs)
+
+    def __del__(self):
+        try: 
+            self._list_manager.delete_temporary_lists()
+        except ReferenceError:
+            pass
+
     @property
     def version(self):
         """
@@ -294,35 +328,10 @@ class Service(object):
         """
         return ResultIterator(self.root, path, params, rowformat, view, self.opener)
 
-    def get_results_list(self, path, params, rowformat, view):
-        """
-        Return a list of the rows of the results
-        ========================================
-
-        This method is called internally by the query objects
-        when they are called to get results. You will not 
-        normally need to call it directly
-
-        @param path: The resource path (eg: "/query/results")
-        @type path: string
-        @param params: The query parameters for this request as a dictionary
-        @type params: dict
-        @param rowformat: One of "dict", "list", "tsv", "csv", "jsonrows", "jsonobjects"
-        @type rowformat: string
-        @param view: The output columns
-        @type view: list
-
-        @raise WebserviceError: for failed requests
-
-        @return: a list of rows of data
-        """
-        rows = self.get_results(path, params, rowformat, view)
-        return [r for r in rows]
-
 class ResultIterator(object):
     
     PARSED_FORMATS = frozenset(["list", "dict"])
-    STRING_FORMATS = frozenset(["tsv", "csv"])
+    STRING_FORMATS = frozenset(["tsv", "csv", "count"])
     JSON_FORMATS = frozenset(["jsonrows", "jsonobjects"])
     ROW_FORMATS = PARSED_FORMATS | STRING_FORMATS | JSON_FORMATS
 
@@ -364,6 +373,7 @@ class ResultIterator(object):
         self.reader = {
             "tsv"         : lambda: FlatFileIterator(con, EchoParser()),
             "csv"         : lambda: FlatFileIterator(con, EchoParser()),
+            "count"       : lambda: FlatFileIterator(con, EchoParser()),
             "list"        : lambda: JSONIterator(con, ListValueParser()),
             "dict"        : lambda: JSONIterator(con, DictValueParser(view)),
             "jsonobjects" : lambda: JSONIterator(con, EchoParser()),
@@ -407,7 +417,7 @@ class FlatFileIterator(object):
 
     def next(self):
         """Return a parsed line of data"""
-        line = self.connection.next()
+        line = self.connection.next().strip()
         if line.startswith("[ERROR]"):
             raise WebserviceError(line)
         return self.parser.parse(line)
@@ -533,8 +543,8 @@ class EchoParser(Parser):
     A result parser that echoes its input
     =====================================
 
-    Parses jsonrow formatted rows into lists
-    of values.
+    Use for parsing situations when you don't
+    actually want to change the data
     """
 
     def parse(self, data):
@@ -609,13 +619,40 @@ class InterMineURLOpener(urllib.FancyURLopener):
         Return a new url-opener with the appropriate credentials
         """
         urllib.FancyURLopener.__init__(self)
+        self.plain_post_header = {
+            "Content-Type": "text/plain; charset=utf-8",
+            "UserAgent": Service.USER_AGENT
+        }
         if credentials and len(credentials) == 2:
             base64string = base64.encodestring('%s:%s' % credentials)[:-1]
             self.addheader("Authorization", base64string)
+            self.plain_post_header["Authorization"] = base64string
             self.using_authentication = True
         else: 
             self.using_authentication = False
 
+    def post_plain_text(self, url, body):
+        o = urlparse(url)
+        con = httplib.HTTPConnection(o.hostname, o.port)
+        con.request('POST', url, body, self.plain_post_header)
+        resp = con.getresponse()
+        content = resp.read()
+        con.close()
+        if resp.status != 200:
+            raise WebserviceError(resp.status, resp.reason, content)
+        return content
+
+    def delete(self, url):
+        o = urlparse(url)
+        con = httplib.HTTPConnection(o.hostname, o.port)
+        con.request('DELETE', url, None, self.plain_post_header)
+        resp = con.getresponse()
+        content = resp.read()
+        con.close()
+        if resp.status != 200:
+            raise WebserviceError(resp.status, resp.reason, content)
+        return content
+
     def http_error_default(self, url, fp, errcode, errmsg, headers):
         """Re-implementation of http_error_default, with content now supplied by default"""
         content = fp.read()
